@@ -1,6 +1,13 @@
 {-# LANGUAGE CPP, NoImplicitPrelude #-}
+#if __GLASGOW_HASKELL__ >= 702
+{-# LANGUAGE Trustworthy #-}
+#endif
+#if MIN_VERSION_base(4,10,0) && !(MIN_VERSION_base(4,12,0))
+{-# LANGUAGE RankNTypes #-}
+{-# LANGUAGE TypeInType #-}
+#endif
 module Prelude.Compat (
-#if MIN_VERSION_base(4,9,0)
+#if MIN_VERSION_base(4,12,0)
   module Base
 #else
   either
@@ -134,6 +141,9 @@ module Prelude.Compat (
 , mappend
 , mconcat
 , mempty
+# if MIN_VERSION_base(4,9,0)
+, (<>)
+# endif
 , maxBound
 , minBound
 , enumFrom
@@ -231,6 +241,9 @@ module Prelude.Compat (
 , Real
 , RealFloat
 , RealFrac
+# if MIN_VERSION_base(4,9,0)
+, Semigroup
+# endif
 , Show
 , Traversable
 
@@ -261,6 +274,9 @@ module Prelude.Compat (
 #if MIN_VERSION_base(4,9,0)
 
 import Prelude as Base
+# if MIN_VERSION_base(4,10,0) && !(MIN_VERSION_base(4,12,0))
+  hiding (($!))
+# endif
 
 #else
 
@@ -301,6 +317,14 @@ import Data.Word
 # endif
 #endif
 
+#if MIN_VERSION_base(4,9,0) && !(MIN_VERSION_base(4,11,0))
+import Data.Semigroup as Base (Semigroup((<>)))
+#endif
+
+#if MIN_VERSION_base(4,10,0) && !(MIN_VERSION_base(4,12,0))
+import GHC.Exts (TYPE)
+#endif
+
 #if !(MIN_VERSION_base(4,9,0))
 -- | A variant of 'error' that does not produce a stack trace.
 --
@@ -309,3 +333,12 @@ errorWithoutStackTrace :: [Char] -> a
 errorWithoutStackTrace s = error s
 {-# NOINLINE errorWithoutStackTrace #-}
 #endif
+
+#if MIN_VERSION_base(4,10,0) && !(MIN_VERSION_base(4,12,0))
+-- | Strict (call-by-value) application operator. It takes a function and an
+-- argument, evaluates the argument to weak head normal form (WHNF), then calls
+-- the function with that value.
+infixr 0 $!
+($!) :: forall r a (b :: TYPE r). (a -> b) -> a -> b
+f $! x = let !vx = x in f vx  -- see #2273
+#endif
